//////////////////////////////////////////////////////////////
//	Squidy - Squid based lolz weapon
//				Feralidragon (18-06-2013)
//
// BUILD 1.00
//////////////////////////////////////////////////////////////

class Squidy expands TournamentWeapon;

var(Inventory) vector WidePlayerViewOffset;
var(Inventory) vector CenterPlayerViewOffset, WideCenterPlayerViewOffset;
var(Inventory) mesh PlayerViewMeshLeft, PlayerViewMeshRight;

var(Weapon) vector AltFireOffset;
var(Weapon) float FireOffsetZAdjustHidden, AltFireOffsetZAdjustHidden;
var(Weapon) float FireOffsetXAdjustHidden, AltFireOffsetXAdjustHidden;

var(RenderOverlays) vector RenderOffsetSelect;
var(RenderOverlays) float AnimMaxFrame;
var(RenderOverlays) rotator RotOffsetSelect;
var(RenderOverlays) vector RenderOffsetFire, RenderOffsetAltFire;
var(RenderOverlays) float AnimMaxFrameFire, AnimMaxFrameAltFire;
var(RenderOverlays) rotator RotOffsetFire, RotOffsetAltFire;

var() int AltDamage;
var() float AltRange;


var bool isPlayingSelect, isInSelAnim;
var byte fireStatus, altFireStatus;


replication
{
	reliable if (Role == ROLE_Authority && bNetOwner)
		WidePlayerViewOffset, CenterPlayerViewOffset, WideCenterPlayerViewOffset;
}


simulated function setHand(float Hand)
{
	setServerHand(Hand);
	if (Hand == 1)
		Mesh = PlayerViewMeshLeft;
	else
		Mesh = PlayerViewMeshRight;
}

function setServerHand(float Hand)
{
    if (Hand == 2)
    {
        PlayerViewOffset.Y = 0;
		WidePlayerViewOffset.Y = 0;
		FireOffset.X = Default.FireOffset.X + FireOffsetXAdjustHidden;
		AltFireOffset.X = Default.AltFireOffset.X + AltFireOffsetXAdjustHidden;
        FireOffset.Y = 0;
		AltFireOffset.Y = 0;
		FireOffset.Z = Default.FireOffset.Z + FireOffsetZAdjustHidden;
		AltFireOffset.Z = Default.AltFireOffset.Z + AltFireOffsetZAdjustHidden;
        bHideWeapon = true;
        return;
    }
    else
        bHideWeapon = false;

    if (Hand == 0)
    {
		if (VSize(CenterPlayerViewOffset) > 0)
			PlayerViewOffset = CenterPlayerViewOffset;
		else
		{
			PlayerViewOffset.X = Default.PlayerViewOffset.X * 0.88;
			PlayerViewOffset.Y = -0.2 * Default.PlayerViewOffset.Y;
			PlayerViewOffset.Z = Default.PlayerViewOffset.Z * 1.12;
		}
		
		if (VSize(WideCenterPlayerViewOffset) > 0)
			WidePlayerViewOffset = WideCenterPlayerViewOffset;
		else
		{
			WidePlayerViewOffset.X = Default.WidePlayerViewOffset.X * 0.88;
			WidePlayerViewOffset.Y = -0.2 * Default.WidePlayerViewOffset.Y;
			WidePlayerViewOffset.Z = Default.WidePlayerViewOffset.Z * 1.12;
		}
    }
    else
    {
        PlayerViewOffset.X = Default.PlayerViewOffset.X;
        WidePlayerViewOffset.X = Default.WidePlayerViewOffset.X;
        PlayerViewOffset.Y = Default.PlayerViewOffset.Y * Hand;
        WidePlayerViewOffset.Y = Default.WidePlayerViewOffset.Y * Hand;
        PlayerViewOffset.Z = Default.PlayerViewOffset.Z;
        WidePlayerViewOffset.Z = Default.WidePlayerViewOffset.Z;
    }
	
    PlayerViewOffset *= 100; //scale since network passes vector components as ints
    WidePlayerViewOffset *= 100; //scale since network passes vector components as ints
    FireOffset.Y = Default.FireOffset.Y * Hand;
    AltFireOffset.Y = Default.AltFireOffset.Y * Hand;
}

simulated function TweenDown()
{
	isPlayingSelect = False;
	isInSelAnim = True;
	PlayAnim('Spin', 3.25, 0.0);
}

simulated function TweenToStill()
{
	isInSelAnim = False;
	if (AnimSequence != 'Spin')
		TweenAnim('Spin', 0.1);
}

function TweenSelect()
{
	isPlayingSelect = True;
	isInSelAnim = True;
    TweenAnim('Spin', 0.1);
}

simulated function PlaySelect()
{
	bForceFire = false;
    bForceAltFire = false;
    bCanClientFire = false;
	isPlayingSelect = True;
	isInSelAnim = True;
    if (!IsAnimating())
        PlayAnim('Spin', 1.2, 0.0);
	if (Owner != None)
		Owner.PlaySound(SelectSound, SLOT_Misc, Pawn(Owner).SoundDampening);  
}

function Projectile ProjectileFire(class<projectile> ProjClass, float ProjSpeed, bool bWarn)
{
local Vector Start, X,Y,Z;
local Pawn PawnOwner;

    PawnOwner = Pawn(Owner);
    Owner.MakeNoise(PawnOwner.SoundDampening);
    GetAxes(PawnOwner.ViewRotation,X,Y,Z);
    Start = Owner.Location + CalcNewDrawOffset() + FireOffset.X * X + FireOffset.Y * Y + FireOffset.Z * Z; 
    AdjustedAim = PawnOwner.AdjustAim(ProjSpeed, Start, AimError, True, bWarn);
	return Spawn(ProjClass,,, Start,AdjustedAim);
}

function Projectile AltProjectileFire(class<projectile> ProjClass, float ProjSpeed, bool bWarn)
{
local Vector Start, X,Y,Z;
local Pawn PawnOwner;

    PawnOwner = Pawn(Owner);
    Owner.MakeNoise(PawnOwner.SoundDampening);
    GetAxes(PawnOwner.ViewRotation,X,Y,Z);
    Start = Owner.Location + CalcNewDrawOffset() + AltFireOffset.X * X + AltFireOffset.Y * Y + AltFireOffset.Z * Z; 
    AdjustedAim = PawnOwner.AdjustAim(ProjSpeed, Start, AimError, True, bWarn);
	return Spawn(ProjClass,,, Start,AdjustedAim); 
}

function Fire(float Value)
{
    if (AmmoType == None && AmmoName != None)
        GiveAmmo(Pawn(Owner));

    if (useAmmoToFire())
    {
		GotoState('NormalFire');
		bPointing = True;
		bCanClientFire = True;
		ClientFire(Value);
		if (bRapidFire || FiringSpeed > 0)
			Pawn(Owner).PlayRecoil(FiringSpeed);
		ProjectileFire(ProjectileClass, ProjectileSpeed, bWarnTarget);
    }
}

simulated function PlayFiring()
{
	fireStatus = 0;
	isInSelAnim = False;
	TweenAnim('Thrust', 0.085);
	PlayOwnedSound(FireSound, SLOT_None, Pawn(Owner).SoundDampening*4.0);
	
}

simulated function bool PlayExtendedFire()
{
	if (fireStatus == 0)
		PlayAnim('Thrust', 2.5, 0.0);
	else if (fireStatus == 1)
		TweenAnim('Spin', 0.2);
	fireStatus++;
	return (fireStatus >= 3);
}

function AltFire(float Value)
{
    if (AmmoType == None && AmmoName != None)
        GiveAmmo(Pawn(Owner));
	
	if (useAmmoToFire(True))
	{
		GotoState('AltFiring');
		bPointing = True;
		bCanClientFire = True;
		ClientAltFire(Value);
		if (bRapidFire || FiringSpeed > 0)
			Pawn(Owner).PlayRecoil(FiringSpeed);
		TraceFire(0.0);
	}
}

function TraceFire(float Accuracy)
{
local vector HitLocation, HitNormal, StartTrace, EndTrace, X, Y, Z;
local actor Other;

	if (Pawn(Owner) == None)
		return;

	Owner.MakeNoise(Pawn(Owner).SoundDampening);
	GetAxes(Pawn(Owner).ViewRotation, X, Y, Z);
	StartTrace = Owner.Location + CalcNewDrawOffset() + FireOffset.Y*Y + FireOffset.Z*Z; 
	EndTrace = StartTrace + Accuracy*(FRand() - 0.5)*Y*1000 + Accuracy * (FRand() - 0.5)*Z*1000;
	AdjustedAim = pawn(Owner).AdjustAim(1000000, StartTrace, 2.75*AimError, False, False);  
	EndTrace += (AltRange * vector(AdjustedAim)); 
	Other = Pawn(Owner).TraceShot(HitLocation, HitNormal, EndTrace, StartTrace);
	ProcessTraceHit(Other, HitLocation, HitNormal, vector(AdjustedAim), Y, Z);
}

function ProcessTraceHit(Actor Other, Vector HitLocation, Vector HitNormal, Vector X, Vector Y, Vector Z)
{
	if (Other != Self && Other != Owner && Other != None)
	{
		Other.PlaySound(Sound'UnrealShare.Gibs.Gib2',, 12.0);
		Other.TakeDamage(AltDamage, Pawn(Owner), HitLocation, vect(0,0,0), AltDamageType);
	}
}

simulated function PlayAltFiring()
{
	altFireStatus = 0;
	isInSelAnim = False;
	PlayOwnedSound(AltFireSound, SLOT_None, Pawn(Owner).SoundDampening*1.5);
	TweenAnim('Swim', 0.05);
}

simulated function bool PlayExtendedAltFire()
{
	if (altFireStatus == 0)
		PlayAnim('Swim', 3.85, 0.0);
	else if (altFireStatus == 1)
		TweenAnim('Spin', 0.2);
	altFireStatus++;
	return (altFireStatus >= 3);
}

function bool useAmmoToFire(optional bool bAltFire)
{
	return (bAltFire || AmmoType.UseAmmo(1));
}

simulated event RenderOverlays( canvas Canvas )
{
local rotator NewRot;
local bool bPlayerOwner;
local int Hand;
local PlayerPawn PlayerOwner;

	if (bHideWeapon || Owner == None)
		return;

	PlayerOwner = PlayerPawn(Owner);
	if (PlayerOwner != None)
	{
		if (PlayerOwner.DesiredFOV != PlayerOwner.DefaultFOV)
			return;
		bPlayerOwner = true;
		Hand = PlayerOwner.Handedness;
		if (Level.NetMode == NM_Client && Hand == 2)
		{
			bHideWeapon = true;
			return;
		}
	}

	if (!bPlayerOwner || PlayerOwner.Player == None)
		Pawn(Owner).WalkBob = vect(0,0,0);

	if (bMuzzleFlash > 0 && bDrawMuzzleFlash && Level.bHighDetailMode && MFTexture != None)
	{
		MuzzleScale = Default.MuzzleScale * Canvas.ClipX/640.0;
		if (!bSetFlashTime)
		{
			bSetFlashTime = true;
			FlashTime = Level.TimeSeconds + FlashLength;
		}
		else if (FlashTime < Level.TimeSeconds)
			bMuzzleFlash = 0;
		if (bMuzzleFlash > 0)
		{
			if (Hand == 0)
				Canvas.SetPos(Canvas.ClipX/2 - 0.5 * MuzzleScale * FlashS + Canvas.ClipX * (-0.2 * Default.FireOffset.Y * FlashO), Canvas.ClipY/2 - 0.5 * MuzzleScale * FlashS + Canvas.ClipY * (FlashY + FlashC));
			else
				Canvas.SetPos(Canvas.ClipX/2 - 0.5 * MuzzleScale * FlashS + Canvas.ClipX * (Hand * Default.FireOffset.Y * FlashO), Canvas.ClipY/2 - 0.5 * MuzzleScale * FlashS + Canvas.ClipY * FlashY);
			Canvas.Style = 3;
			Canvas.DrawIcon(MFTexture, MuzzleScale);
			Canvas.Style = 1;
		}
	}
	else
		bSetFlashTime = false;

	SetLocation(Owner.Location + CalcNewDrawOffset());
	newRot = CalcNewRenderRot();
	if (Hand == 0)
		newRot.Roll = -2 * Default.Rotation.Roll;
	else
		newRot.Roll = Default.Rotation.Roll * Hand;
	setRotation(newRot);
	Canvas.DrawActor(self, false);
}

simulated function rotator CalcNewRenderRot()
{
local rotator R, ROffset;
local Pawn PawnOwner;

	PawnOwner = Pawn(Owner);
	if (PawnOwner != None)
		R = rotator(-vector(PawnOwner.ViewRotation));
	else
		R = Rotation;
	
	//Select Rotation Offset
	if (AnimMaxFrame > 0 && isInSelAnim && AnimSequence == 'Spin')
	{
		ROffset = RotOffsetSelect;
		if (PlayerViewOffset.Y < 0)
			ROffset.Yaw = -ROffset.Yaw;
		
		if (!isPlayingSelect)
			ROffset *= (AnimFrame / AnimMaxFrame);
		else
			ROffset *= ((AnimMaxFrame - AnimFrame) / AnimMaxFrame);
		return rTurn(R, ROffset);
	}
	
	//Fire Rotation Offset
	if (AnimMaxFrameFire > 0 && AnimSequence == 'Thrust')
	{
		ROffset = RotOffsetFire;
		if (PlayerViewOffset.Y < 0)
			ROffset.Yaw = -ROffset.Yaw;
		
		if (AnimFrame < AnimMaxFrameFire / 2)
			ROffset *= (AnimFrame / (AnimMaxFrameFire / 2));
		else
			ROffset *= ((AnimMaxFrameFire - AnimFrame) / (AnimMaxFrameFire/2));
		return rTurn(R, ROffset);
	}
	
	//AltFire Rotation Offset
	if (AnimMaxFrameAltFire > 0 && AnimSequence == 'Swim')
	{
		ROffset = RotOffsetAltFire;
		if (PlayerViewOffset.Y < 0)
			ROffset.Yaw = -ROffset.Yaw;
		
		if (AnimFrame < AnimMaxFrameAltFire / 2)
			ROffset *= (AnimFrame / (AnimMaxFrameAltFire / 2));
		else
			ROffset *= ((AnimMaxFrameAltFire - AnimFrame) / (AnimMaxFrameAltFire/2));
		return rTurn(R, ROffset);
	}
	
	return R;
}

simulated function vector CalcNewDrawOffset()
{
local vector DrawOffset, WeaponBob, CurrentPlayerView, ROffset;
local Pawn PawnOwner;

    PawnOwner = Pawn(Owner);
	if (PlayerPawn(PawnOwner) != None && PlayerPawn(PawnOwner).DefaultFOV >= 100)
		CurrentPlayerView = WidePlayerViewOffset;
	else
		CurrentPlayerView = PlayerViewOffset;
	
	//Select Render Offset
	ROffset = RenderOffsetSelect * 100;
	if (VSize(ROffset) > 0 && AnimMaxFrame > 0 && isInSelAnim && AnimSequence == 'Spin')
	{
		if (PlayerViewOffset.Y < 0)
			ROffset.Y = -ROffset.Y;
		
		if (!isPlayingSelect)
			CurrentPlayerView = CurrentPlayerView + (AnimFrame * ROffset / AnimMaxFrame);
		else
			CurrentPlayerView = CurrentPlayerView + ((AnimMaxFrame - AnimFrame) * ROffset / AnimMaxFrame);
	}
	
	//Fire Render Offset
	ROffset = RenderOffsetFire * 100;
	if (VSize(ROffset) > 0 && AnimMaxFrameFire > 0 && AnimSequence == 'Thrust')
	{
		if (PlayerViewOffset.Y < 0)
			ROffset.Y = -ROffset.Y;
		
		if (AnimFrame < AnimMaxFrameFire / 2)
			CurrentPlayerView = CurrentPlayerView + (AnimFrame * ROffset / (AnimMaxFrameFire / 2));
		else
			CurrentPlayerView = CurrentPlayerView + ((AnimMaxFrameFire - AnimFrame) * ROffset / (AnimMaxFrameFire/2));
	}
	
	//AltFire Render Offset
	ROffset = RenderOffsetAltFire * 100;
	if (VSize(ROffset) > 0 && AnimMaxFrameAltFire > 0 && AnimSequence == 'Swim')
	{
		if (PlayerViewOffset.Y < 0)
			ROffset.Y = -ROffset.Y;
		
		if (AnimFrame < AnimMaxFrameAltFire / 2)
			CurrentPlayerView = CurrentPlayerView + (AnimFrame * ROffset / (AnimMaxFrameAltFire / 2));
		else
			CurrentPlayerView = CurrentPlayerView + ((AnimMaxFrameAltFire - AnimFrame) * ROffset / (AnimMaxFrameAltFire/2));
	}
	
    DrawOffset = ((0.9/PawnOwner.FOVAngle * CurrentPlayerView) >> PawnOwner.ViewRotation);

    if (Level.NetMode == NM_DedicatedServer || (Level.NetMode == NM_ListenServer && Owner.RemoteRole == ROLE_AutonomousProxy))
        DrawOffset += (PawnOwner.BaseEyeHeight * vect(0,0,1));
    else
    {   
        DrawOffset += (PawnOwner.EyeHeight * vect(0,0,1));
        WeaponBob = BobDamping * PawnOwner.WalkBob;
        WeaponBob.Z = (0.45 + 0.55 * BobDamping) * PawnOwner.WalkBob.Z;
        DrawOffset += WeaponBob;
    }
    return DrawOffset;
}

function float RateSelf( out int bUseAltMode )
{
local Pawn P;

	P = Pawn(Owner);
	if (P.Enemy == None)
        return AIRating;
	if (AmmoType.AmmoAmount <=0)
        return -2;
	bUseAltMode = int(VSize(P.Enemy.Location - Owner.Location) < AltRange);
	return (AIRating + FRand() * 0.05);
}

simulated function rotator rTurn(rotator rHeading, rotator rTurnAngle)
{
local vector vForward, vRight, vUpward;
local vector vForward2, vRight2, vUpward2;
local rotator T;
local vector  V;
 
	GetAxes(rHeading,vForward,vRight,vUpward);
	T.Yaw = rTurnAngle.Yaw; 
	V = vector(T);
	vForward2 = V.X*vForward + V.Y*vRight;
	vRight2 = V.X*vRight - V.Y*vForward;
	vUpward2 = vUpward;
	T.Yaw = rTurnAngle.Pitch; 
	V = vector(T);
	vForward = V.X*vForward2 + V.Y*vUpward2;
	vRight = vRight2;
	vUpward = V.X*vUpward2 - V.Y*vForward2;
	T.Yaw = rTurnAngle.Roll; 
	V = vector(T);
	vForward2 = vForward;
	vRight2 = V.X*vRight + V.Y*vUpward;
	vUpward2=V.X*vUpward - V.Y*vRight;
	return OrthoRotation(vForward2, vRight2, vUpward2);
}


state NormalFire
{
	function AnimEnd()
	{
		if (PlayExtendedFire())
			Super.AnimEnd();
	}
}

state ClientFiring
{
    simulated function AnimEnd()
    {
		if (PlayExtendedFire())
			Super.AnimEnd();
    }
}

state AltFiring
{
	function AnimEnd()
	{
		if (PlayExtendedAltFire())
			Super.AnimEnd();
	}
}

state ClientAltFiring
{
    simulated function AnimEnd()
    {
		if (PlayExtendedAltFire())
			Super.AnimEnd();
    }
}


defaultproperties
{
	DrawType=DT_Mesh
    Mesh=LodMesh'UnrealI.Squid1'
	AnimSequence=Spin
	CollisionHeight=16.000000
	CollisionRadius=30.000000
	AutoSwitchPriority=8
	Icon=Texture'Botpack.Icons.IconFlak'
	InventoryGroup=8
	ItemArticle="a"
	ItemName="Squidy"
	MaxDesireability=0.400000
	PickupMessage="You got the Squidy."
	PickupSound=Sound'UnrealI.Squid.death1sq'
	PickupViewMesh=LodMesh'UnrealI.Squid1'
	DrawScale=0.450000
	PickupViewScale=0.450000
	PlayerViewMesh=LodMesh'UnrealI.Squid1'
	PlayerViewMeshLeft=LodMesh'UnrealI.Squid1'
	PlayerViewMeshRight=LodMesh'UnrealI.Squid1'
	PlayerViewOffset=(X=4.00000,Y=-6.000000,Z=-4.500000)
	WidePlayerViewOffset=(X=4.00000,Y=-6.000000,Z=-4.500000)
	PlayerViewScale=0.150000
	RespawnTime=30.000000
	StatusIcon=Texture'Botpack.Icons.IconFlak'
	ThirdPersonMesh=LodMesh'UnrealI.Squid1'
	ThirdPersonScale=0.250000
	AltDamageType=SqPierced
	AltDamage=70
	AltRange=115.000000
	AltFireSound=Sound'UnrealI.Pupae.thumppp'
	AltProjectileClass=None
	AmmoName=Class'SquidyAmmo'
	bInstantHit=False
	bRapidFire=True
	DeathMessage="%o was squided by %k's %w."
	FireSound=Sound'UnrealI.Squid.injur2sq'
	FiringSpeed=1.000000
	MaxTargetRange=4096.000000
	MyDamageType=Squided
	NameColor=(R=0,G=255,B=255)
	PickupAmmoCount=35
	ProjectileClass=Class'SquidyProj'
	SelectSound=Sound'UnrealI.Squid.grab1sq'
	shakemag=0.000000
	shaketime=0.000000
	shakevert=0.000000
	AIRating=0.750000
	AltRefireRate=0.500000
	bAltWarnTarget=True
	bRecommendAltSplashDamage=False
	bRecommendSplashDamage=True
	bSplashDamage=True
	bWarnTarget=True
	RefireRate=1.000000
	Mass=50.000000
	bNoSmooth=False
	
	RenderOffsetSelect=(Z=-4.000000,Y=-10.000000)
	RotOffsetSelect=(Yaw=-12000,Pitch=5000)
	AnimMaxFrame=0.909090
	
	RenderOffsetFire=(X=-5.000000,Z=-1.000000)
	RotOffsetFire=(Pitch=-5000)
	AnimMaxFrameFire=0.947368
	
	RenderOffsetAltFire=(X=2.500000)
	RotOffsetAltFire=(Pitch=1500)
	AnimMaxFrameAltFire=0.952381
}
