//////////////////////////////////////////////////////////////
//	Squidy - Squid based lolz projectile
//				Feralidragon (21-06-2013)
//
// BUILD 1.00
//////////////////////////////////////////////////////////////

class SquidyProj expands Projectile;

var() float WaterSpeedScale, DmgRadius;
var() float SecondsToExplode;
var() sound AmbSwolSnd;
var() int MaxDamage;

var vector locOffset;
var bool bHitWater, bAttached, hasOwner;
var float secCount;

replication
{
	reliable if (Role == ROLE_Authority)
		locOffset, bAttached;
}


function BeginPlay()
{
local vector Dir;

	Dir = vector(Rotation);
	Velocity = speed * Dir;
	PlaySound(SpawnSound,, 12.0);
}

simulated function PostBeginPlay()
{
	Super.PostBeginPlay();
	LoopAnim('Fighter', 2.0);
}

simulated function Tick(float Delta)
{
	if (Physics == PHYS_Falling && Level.NetMode != NM_DedicatedServer)
		SetRotation(rotator(-Velocity));
	else if (Physics == PHYS_None)
	{
		if (hasOwner && (Owner == None || Owner.bDeleteMe || Owner.bHidden || (Pawn(Owner) != None && Pawn(Owner).Health <= 0)))
		{
			BlowUp(Location);
			return;
		}
		
		if (Owner != None)
		{
			SetLocation(Owner.Location + (locOffset >> Owner.Rotation));
			SetRotation(rotator(Location - Owner.Location));
		}
		
		secCount += Delta;
		Fatness = default.Fatness + Byte(115.f * secCount/SecondsToExplode);
		SoundPitch = default.SoundPitch + Byte(50.f * secCount/SecondsToExplode);
		if (secCount >= SecondsToExplode)
			BlowUp(Location);
	}
}

simulated singular function ZoneChange(Zoneinfo NewZone)
{
local waterring w;
local vector Dir;
	
	Dir = Normal(Velocity);
	if (NewZone.bWaterZone && default.LifeSpan == LifeSpan && !bHitWater)
	{
		bHitWater = True;
		Velocity = WaterSpeedScale * speed * Dir;
		return;
	}
	if (NewZone.bWaterZone == bHitWater || default.LifeSpan == LifeSpan) 
		return;
	
	bHitWater = NewZone.bWaterZone;
	if (Level.NetMode != NM_DedicatedServer)
	{
		w = Spawn(class'WaterRing',,,,rot(16384,0,0));
		w.DrawScale = 0.35;
	}
	
	if (NewZone.bWaterZone)
		Velocity = WaterSpeedScale * speed * Dir;
}

simulated function ProcessTouch (Actor Other, Vector HitLocation)
{
	if (bAttached || Other == None || Other == Instigator || Other.IsA('Projectile'))
		return;
	Stick();
	if (Role == ROLE_Authority)
	{
		SetOwner(Other);
		hasOwner = True;
		locOffset = (((Location - Other.Location) - Normal(Location - Other.Location)*CollisionRadius) << Other.Rotation);
	}
}

simulated function Stick()
{
	SetPhysics(PHYS_None);
	PlaySound(ImpactSound,, 12.0);
	PlayAnim('Grab', 2.0);
	if (Role == ROLE_Authority)
		bAttached = True;
}

simulated function AnimEnd()
{
	LoopAnim('Hold', 1.0);
	AmbientSound = AmbSwolSnd;
}

simulated function HitWall(vector HitNormal, actor Wall)
{
	if (bAttached)
		return;
	Stick();
	SetRotation(rotator(HitNormal));
	if (Mover(Wall) != None)
		SetBase(Wall);
}

singular function TakeDamage(int Damage, Pawn EventInstigator, vector HitLocation, vector Momentum, name DamageType)
{
	if (Damage > 0)
	{
		Instigator = EventInstigator;
		BlowUp(HitLocation);
	}
}

function BlowUp(vector HitLocation)
{
	MakeNoise(1.0);
	HurtRadius(int((secCount/SecondsToExplode) * (MaxDamage - Damage) + Damage), DmgRadius, MyDamageType, MomentumTransfer, HitLocation);
	Spawn(Class'SpriteBallExplosion');
	Destroy();
}

defaultproperties
{
	DrawType=DT_Mesh
    Mesh=LodMesh'UnrealI.Squid1'
	DrawScale=0.300000
	CollisionRadius=8.000000
	CollisionHeight=8.000000
	RemoteRole=ROLE_SimulatedProxy
	Physics=PHYS_Falling
	Speed=1500.000000
	MaxSpeed=1800.000000
	LifeSpan=60.000000
	bProjTarget=True
	
	AmbientGlow=96
	bBounce=True
	bNetTemporary=False
	SpawnSound=Sound'UnrealI.Cannon.CannonExplode'
	ImpactSound=Sound'UnrealI.Squid.hit1sq'
	AmbSwolSnd=Sound'UnrealShare.General.Brufly1'
	SoundRadius=64
	SoundVolume=128
	SoundPitch=32
	
	Damage=15
	MaxDamage=95
	DmgRadius=250.000000
	MyDamageType=SquidyDeath
	MomentumTransfer=100000
	
	WaterSpeedScale=0.600000
	SecondsToExplode=2.500000
}